import 'dart:math' as math;

import 'package:knt_bloc/knt_bloc.dart';

import '../model/photo.dart';

class PagingGalleryBloc extends BasePagingBloc<String?, Photo> {
  PagingGalleryBloc() {
    _photos = [];
    on<LoadGalleryPhotosEvent>(
      _handleLoadGalleryPhotosEvent,
      transformer: restartable(),
    );
  }

  late final List<Photo> _photos;

  @override
  Future<BaseResponsePaging<Photo>> pagingInProgress(
    BaseRequestPaging<String?> pagingQuery,
  ) async {
    final photos = _photos.sublist(
      pagingQuery.offset,
      math.min(pagingQuery.offset + pagingQuery.limit, _photos.length),
    );
    await Future<void>.delayed(const Duration(milliseconds: 300));
    return BaseResponsePaging<Photo>(dataList: photos);
  }

  @override
  BasePagingResultState<Photo> pagingStateInstance(
    List<Photo> dataList,
    int offset, {
    String? tag,
  }) =>
      FetchedGalleryPhotosState(dataList, offset, tag: tag);

  Future<void> _handleLoadGalleryPhotosEvent(
    LoadGalleryPhotosEvent event,
    Emitter<BaseState> emit,
  ) async {
    _photos
      ..clear()
      ..addAll(List.from(event.photos));
    emit(LoadedGalleryPhotosState());
  }
}

// region EVENT
class LoadGalleryPhotosEvent extends BaseEvent {
  const LoadGalleryPhotosEvent(this.photos);

  final List<Photo> photos;
}
// endregion

// region STATE
class LoadedGalleryPhotosState extends BaseState {}

class FetchedGalleryPhotosState extends BasePagingResultState<Photo> {
  const FetchedGalleryPhotosState(
    super.dataList,
    super.currentOffset, {
    super.tag,
  });
}
// endregion
