import 'dart:ui';

import '../generated/colors.gen.dart';
import 'widget/image/knt_local_image.dart';

abstract class KntUi {
  static KntUiConfig _shared = const KntUiConfig(
    primaryColor: ColorName.jungleGreen,
    secondaryColor: ColorName.doveGray2,
  );

  static void config({
    required Color primaryColor,
    required Color secondaryColor,
    double? designWidth,
    double? designHeight,
    Color? shimmerColor,
    AbsolutePathConverter? filePathConverter,
    String? assetsErrorPath,
  }) {
    _shared = KntUiConfig(
      primaryColor: primaryColor,
      secondaryColor: secondaryColor,
      designWidth: designWidth,
      designHeight: designHeight,
      shimmerColor: shimmerColor,
      filePathConverter: filePathConverter,
      assetsErrorPath: assetsErrorPath,
    );
  }

  static KntUiConfig get shared => _shared;
}

class KntUiConfig {
  const KntUiConfig({
    required this.primaryColor,
    required this.secondaryColor,
    this.designWidth,
    this.designHeight,
    this.shimmerColor,
    this.filePathConverter,
    this.assetsErrorPath,
  });

  /// Use for design file screen width
  final double? designWidth;

  /// Use for design file screen width
  final double? designHeight;

  /// Usually in used for app bar/tool bar/progress/...
  final Color primaryColor;

  /// Usually in used for sub title text/background/...
  final Color secondaryColor;

  /// Use for shimmer's effect
  final Color? shimmerColor;

  /// Use for convert a relative photo path (usually iOS) into an absolute path
  final AbsolutePathConverter? filePathConverter;

  /// Use for showing an widget which presented any issue/error
  final String? assetsErrorPath;
}
