import 'dart:ui' as ui;

import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';

import '../../model/storage_type.dart';
import 'knt_cached_image.dart';
import 'knt_local_image.dart';

class KntImage extends StatelessWidget {
  const KntImage({
    super.key,
    required this.uri,
    this.width,
    this.height,
    this.child,
    this.placeholder,
    this.color,
    this.borderRadius,
    this.fit,
    this.imageType = StorageType.assets,
    this.converter,
    this.httpHeaders,
  });

  const KntImage.network({
    super.key,
    required this.uri,
    this.width,
    this.height,
    this.child,
    this.placeholder,
    this.borderRadius,
    this.fit,
    this.httpHeaders,
  })  : imageType = StorageType.network,
        color = null,
        converter = null;

  const KntImage.assets({
    super.key,
    required this.uri,
    this.width,
    this.height,
    this.child,
    this.placeholder,
    this.color,
    this.borderRadius,
    this.fit,
  })  : imageType = StorageType.assets,
        converter = null,
        httpHeaders = null;

  const KntImage.file({
    super.key,
    required this.uri,
    this.width,
    this.height,
    this.child,
    this.placeholder,
    this.borderRadius,
    this.fit,
    this.converter,
  })  : imageType = StorageType.file,
        color = null,
        httpHeaders = null;

  final String uri;
  final double? width;
  final double? height;
  final Widget? child;
  final Widget? placeholder;
  final Color? color;
  final BorderRadius? borderRadius;
  final BoxFit? fit;
  final StorageType imageType;
  final AbsolutePathConverter? converter;
  final Map<String, String>? httpHeaders;

  @override
  Widget build(BuildContext context) {
    switch (imageType) {
      case StorageType.network:
        return KntCachedImage(
          url: uri,
          width: width,
          height: height,
          placeholder: placeholder,
          borderRadius: borderRadius,
          fit: fit,
          httpHeaders: httpHeaders,
          child: child,
        );
      case StorageType.assets:
        final image = uri.endsWith('.svg')
            ? SvgPicture.asset(
                uri,
                width: width,
                height: height,
                placeholderBuilder: (_) => placeholder ?? const SizedBox(),
                colorFilter: color == null
                    ? null
                    : ui.ColorFilter.mode(color!, ui.BlendMode.srcIn),
                fit: fit ?? BoxFit.cover,
              )
            : Image.asset(
                uri,
                width: width,
                height: height,
                errorBuilder: (_, __, ___) => placeholder ?? const SizedBox(),
                color: color,
                fit: fit ?? BoxFit.cover,
              );
        final content = SizedBox(
          width: width,
          height: height,
          child: Stack(
            children: [
              Positioned(
                left: 0,
                top: 0,
                right: 0,
                bottom: 0,
                child: image,
              ),
              child ?? const SizedBox(),
            ],
          ),
        );
        if (borderRadius != null) {
          return ClipRRect(
            borderRadius: borderRadius,
            child: content,
          );
        } else {
          return content;
        }
      case StorageType.file:
        return KntLocalImage(
          relativePath: uri,
          converter: converter,
          width: width,
          height: height,
          placeholder: placeholder,
          borderRadius: borderRadius,
          fit: fit,
          child: child,
        );
    }
  }
}
