import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';

import '../config.dart';
import 'knt_expandable_widget.dart';

const _animationDuration = Duration(milliseconds: 400);

class KntFullScreenInverseProgressBar extends StatelessWidget {
  const KntFullScreenInverseProgressBar({
    super.key,
    this.loadingAtFirst = false,
  });

  final bool loadingAtFirst;

  @override
  Widget build(BuildContext context) {
    return KntFullScreenProgressBar(
      loadingAtFirst: loadingAtFirst,
      progressColor: Colors.white,
      backgroundColor: Colors.black54,
    );
  }
}

class KntFullScreenProgressBar extends StatefulWidget {
  const KntFullScreenProgressBar({
    super.key,
    this.loadingAtFirst = false,
    this.progressColor,
    this.backgroundColor,
  });

  final bool loadingAtFirst;
  final Color? progressColor;
  final Color? backgroundColor;

  @override
  State<KntFullScreenProgressBar> createState() =>
      _KntFullScreenProgressBarState();
}

class _KntFullScreenProgressBarState extends State<KntFullScreenProgressBar> {
  late bool _loading;

  @override
  void initState() {
    super.initState();
    _initData();
  }

  @override
  void didUpdateWidget(covariant KntFullScreenProgressBar oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_loading != widget.loadingAtFirst) {
      setState(_initData);
    }
  }

  @override
  Widget build(BuildContext context) {
    return KntExpandableWidget(
      expandedAtFirst: _loading,
      duration: _animationDuration,
      child: Container(
        width: 1.sw,
        height: 1.sh,
        decoration: BoxDecoration(
          color: widget.backgroundColor ?? Colors.transparent,
        ),
        child: KntProgressBar(
          color: widget.progressColor ?? KntUi.shared.primaryColor,
        ),
      ),
    );
  }

  void _initData() {
    _loading = widget.loadingAtFirst;
  }
}

class KntProgressBar extends StatelessWidget {
  const KntProgressBar({
    super.key,
    this.color,
  });

  final Color? color;

  @override
  Widget build(BuildContext context) {
    return Center(
      child: SpinKitCircle(
        size: 48.w,
        color: color ?? KntUi.shared.primaryColor,
      ),
    );
  }
}
