import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import '../../../generated/colors.gen.dart';
import 'knt_selection_popup.dart';

const _popupDuration = Duration(milliseconds: 200);

class KntSimplePicker extends StatelessWidget {
  const KntSimplePicker({
    super.key,
    required this.presentOption,
    this.options = const [],
    this.onPickedOption,
    this.menuMaxHeight,
    this.menuOptionPadding,
    this.needAnimate = false,
  });

  final Widget presentOption;
  final List<String> options;
  final void Function(int, String)? onPickedOption;
  final double? menuMaxHeight;
  final EdgeInsets? menuOptionPadding;
  final bool needAnimate;

  @override
  Widget build(BuildContext context) {
    return KntGeneralPicker<String>(
      presentItem: presentOption,
      options: options,
      onConversion: (option) => option,
      onPickedOption: onPickedOption,
      menuMaxHeight: menuMaxHeight,
      menuOptionHeight: 64.w,
      menuOptionPadding:
          menuOptionPadding ?? EdgeInsets.symmetric(horizontal: 14.w),
      needAnimate: needAnimate,
    );
  }
}

class KntGeneralPicker<T> extends StatelessWidget {
  const KntGeneralPicker({
    super.key,
    required this.presentItem,
    required this.options,
    required this.onConversion,
    this.starterOption,
    this.onPickedOption,
    this.menuMaxHeight,
    this.menuOptionWidth,
    this.menuOptionHeight,
    this.menuOptionPadding,
    this.needAnimate = false,
  });

  final Widget presentItem;
  final List<T> options;
  final String Function(T) onConversion;
  final T? starterOption;
  final void Function(int, T)? onPickedOption;
  final double? menuMaxHeight;
  final double? menuOptionWidth;
  final double? menuOptionHeight;
  final EdgeInsets? menuOptionPadding;
  final bool needAnimate;

  @override
  Widget build(BuildContext context) {
    return KntSelectionPopupPresenter(
      duration: _popupDuration,
      openWithTap: true,
      style: KntSelectionPopupStyle(
        bottomOffsetHeight: 8.w,
        menuOffset: 8.w,
        menuWidth: menuOptionWidth,
        menuMaxHeight: menuMaxHeight,
        menuOptionExtent: menuOptionHeight,
        menuOptionPadding: menuOptionPadding,
        needAnimate: needAnimate,
      ),
      options: List.generate(options.length, (index) {
        final option = options[index];
        return KntSelectionPopupOption(
          title: Text(
            onConversion(option),
            style: TextStyle(
              color: ColorName.tundora,
              fontSize: 14.sp,
              fontWeight: FontWeight.w500,
              height: 16.sp / 14.sp,
            ),
          ),
          trailingIcon: option == starterOption
              ? const Icon(
                  Icons.done_rounded,
                  color: ColorName.jungleGreen,
                )
              : null,
          onPressed: () => onPickedOption?.call(index, option),
        );
      }),
      child: presentItem,
    );
  }
}
