import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';

import '../knt_app_bar.dart';
import '../knt_progress_bar.dart';
import '../knt_status_bar_page.dart';

class KntWebPage extends StatelessWidget {
  const KntWebPage({
    super.key,
    required this.url,
    this.title,
    this.headerColor,
    this.titleColor,
    this.webViewController,
  });

  final String url;
  final String? title;
  final Color? headerColor;
  final Color? titleColor;
  final WebViewController? webViewController;

  static void push(
    BuildContext context, {
    required String url,
    String? titlePage,
    Color? headerColor,
    Color? titleColor,
    WebViewController? webViewController,
  }) =>
      Navigator.of(context).push(
        MaterialPageRoute<void>(
          builder: (BuildContext context) {
            return KntWebPage(
              url: url,
              title: titlePage,
              headerColor: headerColor,
              titleColor: titleColor,
              webViewController: webViewController,
            );
          },
        ),
      );

  @override
  Widget build(BuildContext context) {
    return KntWhiteStatusBarPage(
      page: Scaffold(
        backgroundColor: Colors.white,
        appBar: KntDefaultAppBar(
          titlePage: title,
          backgroundColor: headerColor,
          textColor: titleColor,
        ),
        body: _Content(
          url: url,
          webViewController: webViewController,
        ),
      ),
    );
  }
}

class _Content extends StatefulWidget {
  const _Content({
    required this.url,
    this.webViewController,
  });

  final String url;
  final WebViewController? webViewController;

  @override
  _ContentState createState() => _ContentState();
}

class _ContentState extends State<_Content> {
  late bool _loading;
  late WebViewController _controller;

  @override
  void initState() {
    super.initState();
    _loading = true;
    _controller = widget.webViewController ?? WebViewController()
      ..setJavaScriptMode(JavaScriptMode.unrestricted)
      ..setBackgroundColor(const Color(0x00000000))
      ..setNavigationDelegate(
        NavigationDelegate(
          onPageStarted: _onPageStarted,
          onPageFinished: _onPageFinished,
        ),
      )
      ..loadRequest(Uri.parse(widget.url));
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        WebViewWidget(controller: _controller),
        if (_loading)
          const Align(
            child: KntProgressBar(),
          ),
      ],
    );
  }

  void _onPageStarted(String url) {
    setState(() {
      _loading = true;
    });
  }

  void _onPageFinished(String url) {
    setState(() {
      _loading = false;
    });
  }
}
